import logger from "@egg/logger"
import _, { isObject } from "lodash"
import { Logger } from "winston"

import { baseErrorCard, basePendingCard, baseSuccessCard } from "./component"

type FunctionMap = Record<
  string,
  { xName: string; xAuthor: string; xIcon: string }
>

type CardMap = Record<string, any>

type TempMap = Record<string, string>

class LarkCard<
  TCardMap extends CardMap,
  TTempMap extends TempMap,
  TFunctionMap extends FunctionMap,
> {
  protected requestId: string
  protected funcName: string
  protected functionMap: TFunctionMap
  protected cardMap: TCardMap
  protected tempMap: TTempMap
  protected stringify: boolean
  protected logger: Logger

  constructor(
    funcName: string,
    stringify: boolean,
    requestId: string,
    cardMap: TCardMap,
    tempMap: TTempMap,
    functionMap: TFunctionMap
  ) {
    this.stringify = stringify
    this.requestId = requestId
    this.funcName = funcName
    this.functionMap = functionMap
    this.cardMap = {
      success: baseSuccessCard,
      error: baseErrorCard,
      pending: basePendingCard,
      ...cardMap,
    }
    this.tempMap = tempMap
    this.logger = logger.child({ requestId })
  }

  setFunction(funcName: string) {
    this.funcName = funcName
  }

  setStringify(stringify: boolean) {
    this.stringify = stringify
  }

  child(func: string, stringify?: boolean) {
    return new LarkCard(
      func,
      stringify || this.stringify,
      this.requestId,
      this.cardMap,
      this.tempMap,
      this.functionMap
    )
  }

  genCard(cardKey: keyof TCardMap | object, variables: { [key: string]: any }) {
    const card = isObject(cardKey) ? cardKey : this.cardMap[cardKey]
    if (!card) {
      this.logger.error(`Card ${String(cardKey)} not found`)
      throw new Error(`Card ${String(cardKey)} not found`)
    }
    const finalVariables: Record<string, any> = {
      ...variables,
      requestId: this.requestId,
      ...this.functionMap[this.funcName],
    }
    this.logger.debug(
      `Card ${String(cardKey)} final variables: ${JSON.stringify(finalVariables)}`
    )

    const replaceVariables = (str: string): string => {
      const variablePattern = /\$\{(\w+)\}/g
      const matches = str.match(variablePattern)

      if (matches && matches.length === 1 && matches[0] === str) {
        const variableName = matches[0].slice(2, -1)
        return finalVariables[variableName] || ""
      } else {
        return str.replace(variablePattern, (_, v) => finalVariables[v] || "")
      }
    }

    const traverseAndReplace = (obj: any): any => {
      if (_.isString(obj)) {
        return replaceVariables(obj)
      } else if (_.isArray(obj)) {
        return obj.map(traverseAndReplace)
      } else if (_.isObject(obj)) {
        return _.mapValues(obj, traverseAndReplace)
      }
      return obj
    }

    const content = traverseAndReplace(card)

    // 递归遍历content，遇到key是"value"的Object，将variables注入
    const injectVariables = (obj: { value: any }): any => {
      if (_.isObject(obj) && !_.isArray(obj)) {
        if (_.has(obj, "value") && _.isObject(obj.value)) {
          return {
            ...obj,
            value: { ...obj.value, ...finalVariables },
          }
        }
        return _.mapValues(obj, injectVariables)
      } else if (_.isArray(obj)) {
        return obj.map(injectVariables)
      }
      return obj
    }

    const finalContent = injectVariables(content)

    this.logger.debug(
      `Card ${String(cardKey)} final content: ${JSON.stringify(finalContent)}`
    )
    return this.stringify ? JSON.stringify(finalContent) : finalContent
  }

  genSuccessCard(content: any) {
    return this.genCard("success", { content })
  }

  genErrorCard(content: any) {
    return this.genCard("error", { content })
  }

  genPendingCard(content: any) {
    return this.genCard("pending", { content })
  }

  genTempCard(tempKey: keyof TTempMap, variables: { [key: string]: any }) {
    const tempId = this.tempMap[tempKey]
    if (!tempId) {
      this.logger.error(`Temp ${String(tempKey)} not found`)
      throw new Error(`Temp ${String(tempKey)} not found`)
    }
    const finalVariables: Record<string, any> = {
      ...variables,
      requestId: this.requestId,
      ...this.functionMap[this.funcName],
    }
    this.logger.debug(
      `Temp ${String(tempKey)} final variables: ${JSON.stringify(finalVariables)}`
    )
    const content = {
      type: "template",
      data: {
        config: {
          update_multi: true,
          enable_forward: false,
        },
        template_id: tempId,
        template_variable: finalVariables,
      },
    }
    this.logger.debug(
      `Temp ${String(tempKey)} final content: ${JSON.stringify(content)}`
    )
    return this.stringify ? JSON.stringify(content) : content
  }
}

export default LarkCard
